/*  anatomywizard_v01.js: Batch processing script for 3d files for BrainVoyager QX 2.6 and higher

    Functionality: Create anatomical projects (*.vmr), resample to 1x1x1mm and transform to sagittal orientation.   
    Next version (QX 2.8 and higher): also perform intensity inhomogeneity correction and /or transform to AC-PC and Talairach space; or perform (steps of)
    cortex-based alignment (CBA): aligning curvatures (*.smp) of surface files (*.srf). 
    
    Installation: This batch processing bundle consists of anatomywizard.js, anatomywizard.ui and anatomywizard.dylib, place all in 
    /Documents/BVQXExtensions/Plugins_64/ folder.

    Usage: In the BrainVoyager menu, select 'Plugins' > 'anatomywizard'; then select the required processes via checkboxes on the first tab, configure the parameters (if necessary) for VMR creation and resampling ("isovoxelation") and load the required files (first set the number of projects). Please note that once the files have been loaded, a text file is saved in the directory of the first file, so that later the filenames can just be selected by loading the text file with filenames instead of each file separately. 
    
    Notes: 
    a. Sometimes BrainVoyager does not accept any other files than *.dcm in a directory when creating projects.
    b. If any file that should be created by the anatomywizard with the same filename already exist, they will be overwritten without asking.
    c. When multiple *.vmr projects are created, they should have the same acquisition parameters for using this wizard. Same parameters for the projects are not necessary if the 'Create projects' step is skipped.
    
    Latest update: 
    19-06-2013: v0.1 finished
    20-06-2013: creation date   

    Hester Breman, Brain Innovation, 2013.
*/

var scriptObj = new Object; 
scriptObj.scriptNameForUser     = "anatomywizard"; 
scriptObj.dialogFileName        = "anatomywizard_v01.ui"; 
scriptObj.dialogAccessName      = "batchDlg"; 
BrainVoyagerQX.PrintToLog("anatomywizard started...");

var projfilenames;// = new Array();
var projfilenamessize = 0;
var targetfilenames = new Array();
var targetfilenamessize = 0;
var cancel = false;
const UNKNOWN_SPACE             = 0;
const VOLUMETRIC_SPACE          = 1;
const SURFACE_SPACE             = 2;
const TEXTFILE_FOR_PROJECTCREATION      = 1;
const TEXTFILE_FOR_INHCORRECTION        = 2;
const TEXTFILE_FOR_RESAMPLING           = 3;
const TEXTFILE_FOR_REORIENTATION        = 4;
const TEXTFILE_FOR_NORMALISATION        = 5;
const TEXTFILE_FOR_SPHERE_FROM_RECOSM   = 6;
const TEXTFILE_FOR_CALC_CURVATURE       = 7;
const TEXTFILE_FOR_CORTEXBASEDALIGNMENT = 8;


// global widgets
var startButton, cancelButton;
// widgets on global tab
var chbCreateProjects, chbIsovoxel, chbTrfToSag, chbIIHC, chbACPCTALtrf, chbDoCBA, cmbCBAstep, rbtnIncludeRigidCBA, rbtnNotIncludeRigidCBA;
// widgets on create VMR projects tab
var cmbFileType, spbNrSlices, spbXRes, spbYRes, spbBytesPerPixel, chbBigEndian;
// widgets on volume transformations tab
var cmbIsovoxInterpol;
// widgets on surface tab
var spbSphereRes, chbGroupCurv, chbGroupMesh;

// widgets on files tab
var nroffilesBox, getFileButton, selectFilesNowBox, selectTextFileBox;
var selectFilesNowBox3, selectTextFileBox3, btnGetSRFSMPNames, btnMTCGetVMRName, lblVMRforSRFName, spbNrMTCs; 


// ~~~~~~~~~~~~~~~~~~~~~~~ File I/O ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.getNativeProcessingFiles = function() {

	BrainVoyagerQX.PrintToLog("Get files..."); 
    var filetype = "*.*";
    if (chbCreateProjects.checked == 1) {
        var rawfiletype = cmbFileType.currentText;
        if (rawfiletype == "DICOM") {
            filetype = "*.dcm";
        } else if (rawfiletype == "SIEMENS") {
            filetype = "*.IMA";
        } else if (rawfiletype == "PHILIPS_REC") {
            filetype = "*.REC";    
        } else if (rawfiletype == "GE_I") {
            filetype = "*.*";
        } else if (rawfiletype == "GE_MR") {
            filetype = "*.MR";
        } else if (rawfiletype == "ANALYZE") { 
            filetype = "*.hdr"; 
        } 
    } else {
        filetype = "*.vmr";
    }
    
    // select files manually
    projfilenames = new Array();
    if (selectFilesNowBox.checked == 1) {
    
        var nroffiles = nroffilesBox.value;
        if (nroffiles < 1) BrainVoyagerQX.PrintToLog("Please note: number of selected files is 0.");
        BrainVoyagerQX.PrintToLog("Start selecting: " + nroffiles + " files..."); 
        var filecounter;
        // loop over indicated number of files
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            // get raw data filename or *.vmr filename
            
            var projfilename = this.getFileName(filetype);
            if (projfilename.length > 0) projfilenames.push(projfilename);     

        }
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "input");
        BrainVoyagerQX.PrintToLog("Files selected. If all parameters have been set, the Start button can be clicked.");
        
    // read filenames from textfile    
    } else if (selectTextFileBox.checked == 1) {
    
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetTextFileName");
        BrainVoyagerQX.PrintToLog("Start selecting text file with source filenames..."); 
        BrainVoyagerQX.ExecutePlugin();
        if (this.noErrors()) {
            var textfilename = BrainVoyagerQX.GetPluginStringParameter("Filename");
            BrainVoyagerQX.PrintToLog("Selected text filename: " + textfilename);
            this.readTextFileIntoProjectNamesArray(textfilename, TEXTFILE_FOR_PROJECTCREATION);
        } else {
            success = false;
        }

    }    
}

scriptObj.writeToTextFile = function(projfilenames, kindoffiles, stage, targetfilenames) { 

    var filecounter;
    var success;
    
    // write the textfile to the path of the first file
    var filetype, name; 
    filetype = projfilenames[0].substring((projfilenames[0].length-3), projfilenames[0].length); 
    name = String(this.getSuperPath(projfilenames[0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
   // name = String(this.getPath(projfilenames[0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
    BrainVoyagerQX.PrintToLog("Start writing file: " + name);
    var filenamesfile = new QFile(name);     
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly)); 
    var textstr = new QTextStream(filenamesfile); 
    textstr.writeDouble(projfilenames.length);
    textstr.writeString("\n");
    if ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) || (kindoffiles == TEXTFILE_FOR_INHCORRECTION) ||
        (kindoffiles == TEXTFILE_FOR_RESAMPLING) || (kindoffiles == TEXTFILE_FOR_REORIENTATION)) {
        BrainVoyagerQX.PrintToLog("Write processed anatomy names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter] + "\n");
        }   
    } else {
        BrainVoyagerQX.PrintToLog("Kind of files to write to textfile unknown.");
        success = false;
    }
    filenamesfile.close();
    BrainVoyagerQX.PrintToLog("Wrote " + name + ".");
    return success;
}

scriptObj.readTextFileIntoProjectNamesArray = function(textfilename, kindoffiles) {

    var filecounter;    
    var success = true;
    BrainVoyagerQX.PrintToLog("Start reading: " + textfilename);
    var name = String(textfilename); // circumvention of problem with implicit strings
    var filenamesfile = new QFile(name);     
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
    var textstr = new QTextStream(filenamesfile); 
    var nroffiles = parseInt(textstr.readLine());
    BrainVoyagerQX.PrintToLog("Number of projects: " + nroffiles);
    BrainVoyagerQX.PrintToLog("Kind of projects: " + ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) ?   "project creation/preprocessing" :
                                                    (kindoffiles == TEXTFILE_FOR_INHCORRECTION)     ?   "inhomogeneity corrected" :
                                                    (kindoffiles == TEXTFILE_FOR_RESAMPLING)        ?   "isovoxel" :
                                                    (kindoffiles == TEXTFILE_FOR_REORIENTATION)     ?   "sagoriented" :
                                                    (kindoffiles == TEXTFILE_FOR_NORMALISATION)     ?   "normalisation" :
                                                 //   (kindoffiles == TEXTFILE_FOR_SURFACEPROCESSING) ?   "surface related" :  
                                                    "unknown")); 

    if ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) || (kindoffiles == TEXTFILE_FOR_INHCORRECTION) || (kindoffiles == TEXTFILE_FOR_RESAMPLING) ||
            (kindoffiles == TEXTFILE_FOR_REORIENTATION) || (kindoffiles == TEXTFILE_FOR_NORMALISATION)) { 
        projfilenames = new Array();
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            var filename = textstr.readLine();
            BrainVoyagerQX.PrintToLog("File " + (filecounter+1) + ":\n" + filename);
            projfilenamessize = projfilenames.push(filename);
        } // end for
    } else {
        BrainVoyagerQX.PrintToLog("Kind of files unknown.");
    }
    filenamesfile.close();
    return success;
}

scriptObj.getPath = function(filename) { 

	var start = 0; // start searching from begin 
	var last = filename.lastIndexOf("/");
	var path = filename.substring(start, (last+1)); // now return path without filename
	return path;
}

scriptObj.getSuperPath = function(filename) { 
// getSuperPath(): return 'higher' path including file separator "/"

	var start = 0; // start searching from begin 
	var last = filename.lastIndexOf("/");
	var path = filename.substring(start, last); // now return path without filename
//	BrainVoyagerQX.PrintToLog("Data path: " + path);
	last = path.lastIndexOf("/");
	var path2 = path.substring(start, last+1);
//	BrainVoyagerQX.PrintToLog("Path above data: " + path2);
	return path2;
}
   
scriptObj.getVMRName = function() {

    var vmrname = this.getFileName("*.vmr");
    lblVMRname.text = vmrname; 
}

scriptObj.getFileName = function(filetype) {

    var filename = "";
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFileName");
    BrainVoyagerQX.SetPluginStringParameter("FileType", filetype); 
    BrainVoyagerQX.ExecutePlugin();
    if (this.noErrors()) {
        filename = BrainVoyagerQX.GetPluginStringParameter("Filename");
        BrainVoyagerQX.PrintToLog("Selected: " + filename);
    }
    return filename;
}
    
    
function removeSubstringAfterChar(filename, character) {
	var lastsegmentlength = getlastsubstringlength(filename, character);
	var name = "";
	for (i=0; i<(filename.length-lastsegmentlength); i++) {
		letter = filename[i];
		name += letter;
	}	
	return name;
}


function getlastsubstringlength(filename, character) { // character should be string (within double quotes) 
	var lastsegmentlength, letter, charpos, i;
	for (i=filename.length; i>0; i--) {
		letter = filename[i];
//		if (letter == "_") {
        if (letter == character) {
			charpos = i;					
			break;
		}	
	}
	lastsegmentlength = filename.length - charpos;
	return lastsegmentlength;
}



// ~~~~~~~~~~~~~~~~~~~~~~~ help functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

scriptObj.noErrors = function() {

    var noerror;
    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr != -1) {
        noerror = true;
    } else {
        var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
        BrainVoyagerQX.PrintToLog(errormsg);
        noerror = false;        
    }
    return noerror;
}     

scriptObj.getIsovoxInterpolationDegreeStr = function(index) {

    switch (index) {
        case 1: return "trilinear"; break;
        case 2: return "cubic spline"; break;  
        case 3: return "sinc"; break;
        default: return "not known";
    }
}    
    

// ~~~~~~~~~~~~~~~~~~~~~~~ main processing function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startProcessing = function() {

    var success;
    BrainVoyagerQX.PrintToLog("Start processing...");
    if (projfilenames.length == 0) {
        BrainVoyagerQX.PrintToLog("No input files have been selected. Process cancelled.");
    } else {
    
        if (cmbSelectProcSpace.currentIndex == VOLUMETRIC_SPACE) {
    
            if (chbCreateProjects.checked == 1) {
                this.createProjects();
            } 
            
         /*   if (chbIIHC.checked == 1) {
                this.correctIntensityValuesOfAnatomy();
            }
           */ 
            if (chbIsovoxel.checked == 1) {
                this.resampleVoxelsToIsotropicResolution();
            }

            if (chbTrfToSag.checked == 1) {
                this.transformImageToSagittalOrientation();
            }

     /*       if (chbACPCTALtrf.checked == 1) {
                this.transformImageToACPCandTalairachSpace();
            }
       */     
        } else {
            BrainVoyagerQX.PrintToLog("No processing space selected. Process cancelled.");
        }
    }
    this.batchDlg.close();
} 

 
// ~~~~~~~~~~~~~~~~~~~~~~~ Anatomical preprocessing and transformations ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 
scriptObj.correctIntensityValuesOfAnatomy = function() {
 
 
    if (!cancel) {
    
        BrainVoyagerQX.PrintToLog("Start intensity inhomogeneity correction...");
    
        
        // run IIHC
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            BrainVoyagerQX.PrintToLog("Start correcting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
         //   newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "vmr"; 
            newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "_IIHC.vmr"; //! test
            BrainVoyagerQX.PrintToLog("New name: " + newname);
                                            
            project = BrainVoyagerQX.OpenDocument(this.firstfilename); // add path?
            project.CorrectIntensityInhomogeneities();
            projfilenames[filecounter] = newname;
            project.Close();
         
        } // end filecounter        
        
        BrainVoyagerQX.PrintToLog("Finished inhomogeneity correction.");
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_INHCORRECTION, "inhomogeneitycorrection");
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}

}
 
 
scriptObj.resampleVoxelsToIsotropicResolution = function() {

    if (!cancel) {
    
        // get parameter
       var interpolationdegree = cmbIsovoxInterpol.currentIndex; // check if it starts counting at 0 or 1! => BV starts at 1.
    
        // run resampling
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            BrainVoyagerQX.PrintToLog("Start resampling " + (filecounter+1) + " from:\n" + projfilenames[filecounter] + "\nwith interpolation degree: " + 
                                    this.getIsovoxInterpolationDegreeStr(interpolationdegree));
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + "_ISO.vmr";
            BrainVoyagerQX.PrintToLog("New name: " + newname);
                                            
            project = BrainVoyagerQX.OpenDocument(this.firstfilename); // add path?
            success = project.AutoTransformToIsoVoxel(interpolationdegree, newname);//Interpolation for isovoxelation: 1 - trilinear, 2 - cubic spline (recommended), 3 - sinc
            if (!success) {
                BrainVoyagerQX.PrintToLog("Resampling to isotropic voxels did not succeed.");
            } else {
                projfilenames[filecounter] = newname; // if it does not succeed, leave old filename in array; but this could cause problems later on
            }
            project.Close();
         
        } // end filecounter

        BrainVoyagerQX.PrintToLog("Finished resampling.");
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_RESAMPLING, "isovoxel");
        
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}

}  
 
 
scriptObj.transformImageToSagittalOrientation = function() {

    if (!cancel) {
        
        // apply transformation
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            BrainVoyagerQX.PrintToLog("Start reorienting " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + "_SAG.vmr"; 
            BrainVoyagerQX.PrintToLog("New name: " + newname);
            project = BrainVoyagerQX.OpenDocument(this.firstfilename); 
            success = project.AutoTransformToSAG(newname);
            if (!success) {
                BrainVoyagerQX.PrintToLog("Transformation to sagittal orientation did not succeed.");
            } else {
                projfilenames[filecounter] = newname;// if it does not succeed, leave old filename in array; but this could cause problems later on
            }
            project.Close();
         
        } // end filecounter
 
        BrainVoyagerQX.PrintToLog("Finished rotation to sagittal orientation.");
        this.writeToTextFile(projfilenames, TEXTFILE_FOR_REORIENTATION, "sagoriented");
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}
} 

            	
// ~~~~~~~~~~~~~~~~~~~~~~~ Create VMRs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
scriptObj.createProjects = function() {
    
	// properties of ProjectFactory class
    var currentprojecttype;

    // declare and initialize member properties
	var filetype; 
	var firstfilename;
	var nrslices; 
	var stcprefix; 
	var swapbytes; 	 
	var nrbytes; 
	var savingdir;
	var imgresx; 
	var imgresy; 
    
		BrainVoyagerQX.PrintToLog("Retrieve create project parameters...");
    
        // project-specific parameters
        this.filetype   = cmbFileType.currentText;
        this.nrslices   = spbNrSlices.value;
        this.swapbytes  = chbBigEndian.checked;
        this.nrbytes    = spbBytesPerPixel.value;
        this.imgresx    = spbXRes.value;
        this.imgresy    = spbYRes.value; 
    
        // create VMR
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            BrainVoyagerQX.PrintToLog("Start creating project " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + "vmr"; 
            BrainVoyagerQX.PrintToLog("New name: " + newname);
                                            
            var projparams = "Project parameters:\n";
            projparams += "\tFiletype: " + this.filetype + "\n";
            projparams += "\tFirst file name: " + this.firstfilename + "\n";
		    projparams += "\tNumber of slices: " + this.nrslices + "\n";
		    projparams += "\tSwap bytes: " + this.swapbytes + "\n";
            projparams += "\tNumber of bytes: " + this.nrbytes + "\n";
            projparams += "\tSaving directory: " + this.savingdir + "\n";
            projparams += "\tSize of individual image (x-axis): " + this.imgresx + "\n";
            projparams += "\tSize of individual image (y-axis): " + this.imgresy + "\n";
            BrainVoyagerQX.PrintToLog(projparams);

            project = BrainVoyagerQX.CreateProjectVMR(this.filetype, this.firstfilename, this.nrslices, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes);
            project.SaveAs(newname);
            projfilenames[filecounter] = newname;
            project.Close();
         
        } // end filecounter
    this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "projectcreation");
} 



// ~~~~~~~~~~~~~~~~~~~~~~~ main ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 

    this.batchDlg.windowTitle = "anatomywizard"; 
    
    // ~ global widgets
    cmbSelectProcSpace          = BrainVoyagerQX.FindChild("cmbSelectProcSpace");
    
    // native space
    chbCreateProjects           = BrainVoyagerQX.FindChild("chbCreateProjects");
    chbIsovoxel                 = BrainVoyagerQX.FindChild("chbIsovoxel");
    chbTrfToSag                 = BrainVoyagerQX.FindChild("chbTrfToSag");
    chbIIHC                     = BrainVoyagerQX.FindChild("chbIIHC");
    chbACPCTALtrf               = BrainVoyagerQX.FindChild("chbACPCTALtrf");
    
    // surface space
    chbDoCBA                = BrainVoyagerQX.FindChild("chbDoCBA");
    cmbCBAstep              = BrainVoyagerQX.FindChild("cmbCBAstep");
    rbtnIncludeRigidCBA     = BrainVoyagerQX.FindChild("rbtnIncludeRigidCBA");
    rbtnNotIncludeRigidCBA  = BrainVoyagerQX.FindChild("rbtnNotIncludeRigidCBA");
    
    // ~ widgets on create projects tab
    cmbFileType             = BrainVoyagerQX.FindChild("cmbFileType");
    chbBigEndian            = BrainVoyagerQX.FindChild("chbBigEndian"); 
    spbNrSlices             = BrainVoyagerQX.FindChild("spbNrSlices");
    spbXRes                 = BrainVoyagerQX.FindChild("spbXRes");
    spbYRes                 = BrainVoyagerQX.FindChild("spbYRes"); 
    spbBytesPerPixel        = BrainVoyagerQX.FindChild("spbBytesPerPixel");
    
    // ~ widgets on volume transformations tab
    cmbIsovoxInterpol       = BrainVoyagerQX.FindChild("cmbIsovoxInterpol");
    
    // ~ widgets on CBA tab
    spbSphereRes            = BrainVoyagerQX.FindChild("dspbMTCFrom");
    chbGroupCurv            = BrainVoyagerQX.FindChild("chbGroupCurv");
    chbGroupMesh            = BrainVoyagerQX.FindChild("chbGroupMesh");
    
    // ~ widgets on files tab
    // volumetric space
    getFileButton       = BrainVoyagerQX.FindChild("btnGetFiles");
    nroffilesBox        = BrainVoyagerQX.FindChild("spnBoxNrOfFiles");
    selectFilesNowBox   = BrainVoyagerQX.FindChild("rbtnFileDialog");
    selectTextFileBox   = BrainVoyagerQX.FindChild("rbtnTextFile");
    // surface space
    selectFilesNowBox3      = BrainVoyagerQX.FindChild("rbtnFileDialog_4");
    selectTextFileBox3      = BrainVoyagerQX.FindChild("rbtnTextFile_4");    
    btnGetSRFSMPNames       = BrainVoyagerQX.FindChild("btnGetSRFSMPNames"); 
    lblVMRforSRFName        = BrainVoyagerQX.FindChild("lblVMRforSRFName"); 
    btnVMRforSRFName        = BrainVoyagerQX.FindChild("btnVMRforSRFName"); 
    spbNrMTCs               = BrainVoyagerQX.FindChild("spbNrMTCs"); 

    // connect widgets to functions
	this.batchDlg.btnStart.clicked.connect(this, this.startProcessing); // might need to go to top of function
    getFileButton.clicked.connect(this, this.getNativeProcessingFiles);
  //  btnGetMTCnames.clicked.connect(this, this.getSurfaceProcessingFiles);    
    btnVMRforSRFName.clicked.connect(this, this.getVMRName);
    
    
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
